#include "ukr_matlab.h"
   


void mexFunction(int nlhs, mxArray *plhs[], int nrhs,const mxArray *prhs[]) {
UKR_Model model;
UKR_Workspace W;
double *GX,*GXt;
double pen;

   if (nrhs<2) mexErrMsgTxt("Not enough arguments.");
   if (nrhs>3) mexErrMsgTxt("Too many arguments.");
      
   model = ukr_model_from_matlab(prhs[1]);
   
   if (model.N==0) mexErrMsgTxt("Model contains neither valid 'X','Y' nor 'G' -> cannot determine N.");   
   
   if (model.q!=0 && model.q!=mxGetM(prhs[0])) mexErrMsgTxt("Dimensions of X do not match model.q.");
   model.q=mxGetM(prhs[0]);
   
   if (model.N==mxGetN(prhs[0])) {
      model.Xt=model.X=mxGetPr(prhs[0]);
      model.Nt=model.N;
   } else if (model.N+model.Nt == mxGetN(prhs[0])) {
      model.X=mxGetPr(prhs[0]);
      model.Xt=model.X + model.q*model.N;
   } else {
      mexErrMsgTxt("Dimensions of X do not match model.");
   }
      
   if (nrhs>2) {
      model.eta = mxGetScalar(prhs[2]);
   }
      
   if (!ukr_aux_alloc_workspace(model,&W)) mexErrMsgTxt("Sorry, out of memory.");
   
   if (nlhs>1) {
      if (model.X==model.Xt) {
         plhs[1]=mxCreateDoubleMatrix(model.q,model.N,mxREAL);
         GX=GXt=mxGetPr(plhs[1]);
      } else {
         plhs[1]=mxCreateDoubleMatrix(model.q,(model.N + model.Nt),mxREAL);
         GX=mxGetPr(plhs[1]);
         GXt=GX+model.q*model.N;
      }
   } else {
      GX=GXt=NULL;
   }
   
   pen=ukr_densvar_penalty(GX,GXt,model,&W);
   
   plhs[0]=mxCreateDoubleScalar(pen);
   
   ukr_aux_free_workspace(W);
}
