#include "ukr_matlab.h"

void mexFunction(int nlhs, mxArray *plhs[], int nrhs,const mxArray *prhs[]) {
   UKR_Model model;
   UKR_Workspace W;
   double err;
   int forceCV;
   double *grad;
   double *sX;
   const double *s;

   if (nrhs<3) mexErrMsgTxt("Not enough arguments.");
   if (nrhs>4) mexErrMsgTxt("Too many arguments.");
   
   model = ukr_model_from_matlab(prhs[1]);
   
   if (model.Y==NULL && model.G==NULL) mexErrMsgTxt("Model contains neither valid 'Y' nor 'G'");   
   
   
   s = mxGetPr(prhs[0]);
   if (model.q!=0 && model.q!=mxGetM(prhs[0])) mexErrMsgTxt("Dimension of s does not match model.q.");
   model.q=mxGetM(prhs[0]);
   
   if (model.q!=mxGetM(prhs[2])) mexErrMsgTxt("Dimensions of X do not match s.");
   
   if (model.N==mxGetN(prhs[2])) {
      sX = (double *) mxMalloc(model.q*model.N*sizeof(double));
      multDiagMatrix(sX,s,mxGetPr(prhs[2]),model.q,model.N);
      model.Xt=model.X=sX;
      model.Nt=model.N;
   } else if (model.N+model.Nt == mxGetN(prhs[2])) {
      sX = (double *) mxMalloc(model.q*(model.N+model.Nt)*sizeof(double));
      multDiagMatrix(sX,s,mxGetPr(prhs[2]),model.q,model.N+model.Nt);
      model.X=sX;
      model.Xt=sX + model.q*model.N;
   } else {
      mexErrMsgTxt("Dimensions of X do not match model.");
   }
   
   forceCV=0;
   
   if (nrhs>3) {
      double cv = mxGetScalar(prhs[3]);
      if (cv!=0.0) {
         /* overwrite model.Mask, use LOO-CV */      
         forceCV=1;
      } else {
         /* overwrite model.Mask, no CV */
         model.MaskIs=model.MaskJs=NULL;
      }
   }
   
   if (!ukr_aux_alloc_workspace(model,&W)) mexErrMsgTxt("Sorry, out of memory.");
   
   plhs[0]=mxCreateDoubleMatrix(1,1,mxREAL);
     
   if (nlhs>1) {
      plhs[1]=mxCreateDoubleMatrix(model.q,1,mxREAL);
      grad=mxGetPr(plhs[1]);
   } else {
      grad=NULL;
   }
   
   err = ukr_errgrad_scale(grad,s,model,&W,forceCV);
   
   *(mxGetPr(plhs[0]))=err;
   
   mxFree(sX);
   ukr_aux_free_workspace(W);   
}

