#include "ukr_matlab.h"
#include "specialmath.h"

void mexFunction(int nlhs, mxArray *plhs[], int nrhs,const mxArray *prhs[]) {
UKR_Model model;
UKR_Workspace W;
UKR_G_Struct gs;
const double *x0;
double *x;
int n;

   if (nrhs<3) mexErrMsgTxt("Not enough arguments.");
   if (nrhs>3) mexErrMsgTxt("Too many arguments.");
   
   model = ukr_model_from_matlab(prhs[1]);

   if (model.X==NULL) mexErrMsgTxt("Model does not contain 'X'");
   if (model.Y==NULL && model.G==NULL) mexErrMsgTxt("Model does contain neither 'Y' nor 'G'");
   
   gs = ukr_g_struct_from_matlab(prhs[0],model.d,model.N);
   
   x0 = ukr_mx_get_matrix(prhs[2],model.q,1);
   if (x0==NULL) mexErrMsgTxt("x0 must be a 'q x 1' vector (latent space).");   
   
   plhs[0]=mxCreateDoubleMatrix(model.q,1,mxREAL);
   x=mxGetPr(plhs[0]);
   
   ukr_aux_alloc_workspace(model,&W);
   
   for (n=0;n<model.q;n++) x[n]=x0[n];
   
   if (-1==ukr_g_opt_denscon(x,model,&W,gs)) {
      mexErrMsgTxt("Invalid initilazation.");
   }
   ukr_aux_free_workspace(W);
}

