function [err,grad] = ukr_aux_eg_scale(X, Xt, T, model, Mask)
% [err,grad] = ukr_aux_eg_scale(X, Xt, T, model, Mask)
%
% Normally you don't call this function directly.

[q,N]=size(X);
Nt = size(Xt,2);

if ~isfield(model,'degree') | model.degree==0

   [K,sumK,P] = ukr_kp_dists(ukr_aux_distances(X,Xt),model,Mask);
   iSumK=sumK.^(-1);
   iSumK(sumK==0)=0;
   B = K.*repmat(iSumK,N,1);
   P = P.*repmat(iSumK,N,1);

   if nargout>1
      [err,M] = ukr_aux_err_m(B,model,T);

      Q = P.*(M - repmat(sum(B.*M),N,1));

      r = (X.*X)*sum(Q,2) - 2*sum((X*Q).*Xt,2) + sum((Xt.*Xt).*repmat(sum(Q,1),q,1),2);
      grad = -(1/Nt)*r;
   else
      err = ukr_aux_err_m(B,model,T);
   end

   err = err/Nt;
else
   switch(model.degree)
      case 1
      case 2
      case 3
         if q~=2
            error 'Full quadratic model (degree=3) only implemented for q=2';
         end
      otherwise 
         error 'model.degree must be between 0 and 3 (q=2)';
   end

   err = 0;
   grad = zeros(q,1);

   for n=1:Nt

      Xn = X - repmat(Xt(:,n),1,N);

      [errn,XQ_T] = ukr_aux_XQ_T(Xn, model, Mask(:,n),T(:,n));
      err = err+errn;
      
      grad = grad + sum(Xn.*XQ_T,2);
   end
   
   grad = -(1/Nt)*grad;
   err = err/Nt;
end
