function [err,M] = ukr_aux_err_m(B,model,T);
%function [err,M] = ukr_aux_err_m(B,model,Y);
%function [err,M] = ukr_aux_err_m(B,model,Phi);
%
% B      vector / matrix of basis functions
% model  UKR model
% Y      data vector / matrix to project (may be model.Y itself)
% Phi    mixed mercer kernel matrix (or model.G itself)    

if isfield(model,'G')
   % in this case, T gives us the column(s) which we're evaluating
   M=model.G*B-model.G(:,T);
   
   if length(T)==1
      err = full(sum(B.*M)) - M(T);
   else % if model.degree = 0, T might also be 1:N
      err=full(sum(sum(B.*M)))-sum(diag(M));
   end
   if nargout>1
      M = 2*M;
   end
else
   [d,N]=size(model.Y);
   
   YBmT = model.Y*B-T;
   if isfield(model,'loss')
      [err,G]=ukr_loss_function(YBmT,model);
      if nargout>1
         M = model.Y'*G;
      end
   else
      err = norm(YBmT,'fro')^2;
      if nargout>1
         M = 2*model.Y'*YBmT;
      end
   end
end   
