function [b, J, dens] = ukr_bJ(x, model)
% function [b, J, dens] = ukr_bJ(x, model)
%
% Computes normalized vector of basis functions and the
% corresponding Jacobi matrix  J_ij = db_i / dx_j
%
% get dimensions:
[q n] = size(x);
[qm N]=size(model.X);

if n~=1 | qm~=q; error '"x" must be "q x 1"'; end

% compute distance and kernel matrices
[k,sumk,p] = ukr_kp_dists(ukr_aux_distances(model.X,x),model);

if ~isfield(model,'degree') | model.degree==0
   if sumk>0
      b = k/sumk;
      p = p/sumk;
   end

   if issparse(b)
      J = model.X'.*repmat(p,1,q) - sparse(p*x') + sparse(b*sum(p)*x') - b*sparse(model.X*p)';
   else
      J = model.X'.*repmat(p,1,q) - p*x' + b*sum(p)*x' - b*(model.X*p)';   
   end
else
   [b,J] = ukr_aux_lq_bJ(x,model,k,p);
end

if nargout>2
   dens=sumk/N;
end
