%UKR_DEMO Simple MATLAB script to present and explain basic features of the UKR toolbox.

%create and display 'noisy half-circle' data-set and test-set:
fprintf(1, 'Creating two dimensional observable space data set.\n');
N = 100; Neval = 15; d = 2;
t = rand(1, N)*pi;
Noise = randn(d, N)*0.1;
DATA = [sin(t) ; cos(t)];
DATA = DATA + Noise;

teval = rand(1,Neval)*pi;
Noise = randn(d,Neval)*0.1;
TESTDATA = [sin(teval) ; cos(teval)];
TESTDATA = TESTDATA + Noise;

fig = figure; set(fig, 'Position', [471    26   800   400]);
subplot(1, 3, 1);
hold on;
scatter(DATA(1, :), DATA(2, :), 10, t, 'o', 'filled');
axis([-1.5 1.5 -1.5 1.5]); axis square;
drawnow;


%create and train a one-dimensional ukr_model (use default initialization ):
fprintf(1, 'Creating and training one dimensional UKR model.\n');
model = ukr_model('Y', DATA, 'q', 1);      % create valid ukr_model structure

model.display=1;
model.X = ukr_train(model);    % Train the ukr model 
model.eta = ukr_eta(model);
                
model.display=0;

fprintf(1, 'Visualizing solution.\n');
subplot(1, 3, 2);
scatter(model.X, ones(1, N), 15, t);
axis auto; axis square;

% Visualize regression function:
X_ = linspace(min(model.X),max(model.X),300);

% Apply regression function on resulting data set
Y_ = ukr_f(X_, model);      

subplot(1, 3, 1);
h=plot(Y_(1, :), Y_(2, :),'k-');
set(h,'LineWidth',2);
drawnow;

%Compute and visualize test set projections:
fprintf(1, 'Computing test set projections.\n');
[YNear,XNear] = ukr_nr(TESTDATA,model);
XTest = ukr_g(TESTDATA,model);
Yproj = ukr_f(XTest,model);
%Yproj = ukr_proj(TESTDATA,model);

subplot(1, 3, 3);
hold on;
plot(Y_(1, :), Y_(2, :), 'k-');
scatter(TESTDATA(1, :), TESTDATA(2, :), '.');
scatter(Yproj(1, :), Yproj(2, :), 'kx');
for ii = 1 : Neval; line([TESTDATA(1, ii), Yproj(1, ii)], [TESTDATA(2, ii), Yproj(2, ii)]); end;
axis([-1.5 1.5 -1.5 1.5]); axis square;

