function [err,grad] = ukr_errgrad_scale(s,model,Xu,cv)
% [err, grad] = ukr_errgrad_scale(s,model,X [,cv]) 
%
% Compute mean squared reconstruction error and its gradient w.r.t. 
% the scale parameter  s.
%
% The flag "cv" determines if cross-validation is used
% If not given, model.Mask determines the kind of cross-validation
%   cv=0 : do not use cross-validation (overrides model.Mask)
%   cv~=0: use leave-one-out cross-validation (overrides model.Mask)
%
% Normally you don't call this function directly.

error(nargchk(3, 4, nargin))

[q,Na]=size(Xu);

if isfield(model,'G')
   N=size(model.G,2);
   T=1:N; % this is an implementation trick
   Nt=N;      
   if N~=Na
      error 'Dimensions of X and model.G do not match';
   end
   plain=1;
else
   if isfield(model,'Yt')
      T = model.Yt;
      [d,Nt]=size(T);
      [d2,N]=size(model.Y);
      if d~=d2
         error 'Dimension of model.Y and model.Yt do not match.';
      end
      if Na~=N+Nt
         error 'Dimensions of X=(Xs,Xt) does not match (model.Y,model.Yt)';        
      end
      plain=0;
   else
      T=model.Y;
      N=size(T,2);
      Nt=N;
      if N~=Na
         error 'Dimensions of X=(Xs,Xt) does not match model.Y';
      end
      plain=1;
   end
end

if nargin>3
   if cv
      if N~=Nt
         error 'LOO-CV cannot be applied in this case.'
      end
      Mask=speye(N);
   else
      Mask = sparse([],[],[],N,Nt,0);
   end
else
   if isfield(model,'Mask')
      if ~isequal(size(model.Mask),[N Nt])
         error 'Given CV-Mask does not match given latent variable matrix';
      end
      Mask = model.Mask;
   else
      Mask = sparse([],[],[],N,Nt,0);
   end
end

if plain 
   X = diag(s)*Xu;
   Xt = X;
else
   X = diag(s)*Xu(:,1:N);
   Xt = diag(s)*Xu(:,(N+1):end);
end

[err,grad] = ukr_aux_eg_scale(X,Xt,T,model,Mask);

grad = grad./s;
