function [Y,dens] = ukr_f(X, model)
% ukr_f Evaluate ukr function for one or several latent space elements.
% [Y] = ukr_f(X, model)
%
% Inputs:
%   X      -  (qxN)-Matrix of latent space elements (columnwise).
%   model  -  Valid ukr_model structure.
%
% Output:
%   Y      -  Result of applying the regression function as implicitly 
%             defined by the input ukr model to each element in X.
%   dens   -  row vector of densities at latent positions X             
%
% See also ukr_g.
%
% Notice : If using Kernel UKR, you should not call this function as
% it calculates within the wrong space. Instead use
%     B=ukr_b(X,model);
% to retrieve the mixing weights of the models feature space support
% vectors.

error(nargchk(2, 2, nargin))

% get dimensions:
[d N] = size(model.Y);

% get vector of basis function responses:

if nargout>1
   [b,dens] = ukr_b(X, model);
else
   b = ukr_b(X,model);
end

% compute output y as linear combination of training sample elements:

Y = model.Y * b;

