function x = ukr_g_opt_denscon(y,model,x0)
% function x = ukr_g_opt_denscon(y,model,x0)
%
% Nonlinear least squares optimization (+ nonlinear density constraint)
% used for projecting data vectors onto UKR manifolds in ukr_g
%
% Normally, you won't use this function directly

KKT_SENS=0.99;
CON_SENS=-0.00001;
GRAD_SENS=1e-8;
MAX_ITER=50;
MIN_STEP=0.000001;

x=x0;

[f,gf,Hf]=ukr_g_errgradhess(x,model,y);
[c,gc]=ukr_g_congrad(x,model);
ngc=norm(gc);
ngf=norm(gf);

if c>0 
	error 'invalid init.';
end

iter = 0;
q = length(x);
lambda=0.1*f;

if isfield(model,'display') & model.display
	disp=1;
else
	disp=0;
end


while iter<MAX_ITER

	if disp
		fprintf(1,'[%3i] %f\t%f\t%f\n',iter,f,c,lambda);	
	end
   
   if ngf < GRAD_SENS
%         fprint(1,'Mimimum found');
      break;
   end
		
	if c>CON_SENS & ngc > 0
   	measure = -gc'*gf/(ngc*norm(gf));
		
		if measure>KKT_SENS
%				fprintf(1,'KKT point\n');
			break
		end
	end
	
	H = Hf + lambda*eye(q);
	dx = pdqp_slc(H,gf,gc,-c);
 
	pred=0.5*dx'*H*dx + gf'*dx;
	
	oldx = x;
	x = x + dx;
	
	[c,gc]=ukr_g_congrad(x,model);
	ngc = norm(gc);
	
	if c>0
      iterC = 0;
		while c>0 & iterC<12
         iterC=iterC+1;
			if ngc == 0 
				x = 0.5*(x+oldx);
			else
				x = x - c*gc/(ngc^2);
			end
			[c,gc]=ukr_g_congrad(x,model);
			ngc=norm(gc);
		end
	end
	
	if norm(x-oldx)<MIN_STEP break; end
	
	iter=iter+1;
	
	oldf = f;
	
	[f,gf,Hf]=ukr_g_errgradhess(x,model,y);
   ngf=norm(gf);   
	
	if f>oldf+0.5*pred
		lambda=lambda*5;
	else
		lambda=lambda*0.9;
	end
end
		
		
function [x,lambda] = pdqp_slc(A,a,b,b0)
% function x = pdqp(A,a,b,b0)
% Find minimum x of x'Ax + a'x  s.t. b'x<=b0
% A must be positive definite

Aa = A\a;

x = -Aa;

if b'*x <= b0 
	lambda=0;
	return; 
end

x0 = b*(b0/(b'*b));

Ab = A\b;

lambda=(b'*Ab)\(b'*Aa);

x = x0 + lambda*Ab - Aa;	
