function [s,err] = ukr_initscale(X,model)
% function [s,err] = ukr_initscale(X,model)
%
% Find optimization scale factors for an initial set
% of latent variables.

n=5;

disp = 0;
if isfield(model,'display') 
   disp=model.display;
end

[q,N]=size(X);

h_rot = std(X,1,2)*N^(-1/(q+4));

if strcmp(model.kernel_l,'Quartic')==0
	h_rot = h_rot*(2.0362/0.7764)^q;
end

s=h_rot.^(-1);

M=10;

err=zeros(M,1);
fak=zeros(M,1);
for k=1:M
	fak(k)=2^(k-4);
   if isfield(model,'Mask')
      err(k)=ukr_errgrad_scale(fak(k)*s,model,X);
   else
      err(k)=ukr_errgrad_scale(fak(k)*s,model,X,1);
   end
   if disp
      fprintf(1,'%f\n',err(k));
   end
end

[merr,ind]=min(err);

if ~isnan(merr)
   if isfield(model,'Mask')
%      [s,err]=ukr_opt_unc(@ukr_errgrad_scale,fak(ind)*s,10,model,X);            
      [s,err]=fminsearch(@ukr_errgrad_scale,fak(ind)*s,[],model,X);            
   else
%      [s,err]=ukr_opt_unc(@ukr_errgrad_scale,fak(ind)*s,10,model,X,1);      
      [s,err]=fminsearch(@ukr_errgrad_scale,fak(ind)*s,[],model,X,1);      
   end
   s=abs(s);
else
   s=0;
   err=realmax;
end

