function [K,sumK,P] = ukr_kp_dists(DX, model, Mask)
%function [K,sumK,P] = ukr_kp_dists(DX, model, Mask)

error(nargchk(2, 3, nargin))

if nargin<3; 
   Mask=[]; 
else
   if ~isempty(Mask) & ~isequal(size(Mask),size(DX))
      error 'ukr_kp_dists was called with optional "Mask" of incorrect size';
   end
end

[M,N] = size(DX);

switch model.kernel_l
   case 'Gaussian';

      K = exp(-0.5*DX);
      if ~isempty(Mask)
         K(logical(Mask))=0;
      end
      
      if nargout>2
         P = K;
      end
      
   case 'Quartic';

      P = 1 - DX; % Epanechnikov
      P(P<0) = 0;
      if ~isempty(Mask)
         P(logical(Mask)) = 0;
      end
      
      P = sparse(P);
      K = P.^2; % Quartic
      
      if nargout>2
         P = 4*P;
      end
          
   case 'Triweight';
      E = 1 - DX; % Epanechnikov
      E(E<0) = 0;
      if ~isempty(Mask)
         E(logical(Mask)) = 0;
      end
      E=sparse(E);
      if nargout>1
         P = E.^2;
      end
      K = E.^3; % Triweight
      
      if nargout>2
         P = 6*P;
      end
end

if nargout>1
   sumK=full(sum(K));      
end

