function T = ukr_mercer(model,Y) 
% D = ukr_mercer(model)
% calculates the matrix of dot products in feature space
%     D(i,j) = mercer(y_i,y_j)     (feature space gramian)
%
% T = ukr_mercer(model,Z)
% calculates matrix of dot products in feature space
%     T.D(i,j) = mercer(y_i,z_j)
%     T.d(i)   = mercer(z_i,z_i)
%
% where 
%    mercer : function pointed to by model.kernel_o 
%      y_i  : column i of model.Y (i-th training vector)
%      z_i  : column i of Z       (i-th test vector

error(nargchk(1, 2, nargin)) 

if nargin>1
    [d M]=size(Y);
    [d N]=size(model.Y);
    
    if ~isfield(model,'kernel_o')
        error('No mercer kernel (model.kernel_o) given !');
    end

    T.D=zeros(N,M);
    T.d=zeros(1,M);
    for n=1:N
        for m=1:M
            T.D(n,m)=feval(model.kernel_o,Y(:,m),model.Y(:,n));
        end
    end
    for m=1:m
       T.d(m)=feval(model.kernel_o,Y(:,m),Y(:,m));
    end
else
    [d N]=size(model.Y);
    T=zeros(N,N);
    for n=1:N
        for m=1:(n-1)
            T(m,n)=T(n,m);
        end
        for m=n:N
            T(m,n)=feval(model.kernel_o,model.Y(:,m),model.Y(:,n));
        end
    end
end
