function X = ukr_pca(model)
% ukr_pca : Calculate PCA/KPCA initialization
% X = ukr_pca(model)
%
% X is set to the first model.q principal components
% of the model's data, scaled down to a total variance
% of 1. This leads to good initial values for further
% optimizing the latent variable positions if the data
% distribution is nearly linear.
%
% If Kernel UKR is used ( if model.G exists) Kernel PCA
% is automatically run instead of normal PCA, so you 
% get the first q principal components in feature space.

if isfield(model,'G')
    % perform Kernel PCA (linear PCA in feature space)
    [N,N]=size(model.G);
    opts.disp=0;

    % center in feature space
    sg=sum(model.G)/N;
    K=model.G-repmat(sg,N,1)-repmat(sg',1,N)+repmat(sum(sg)/N,N,N);
    % get eigenvalues and coeff. for eigenvectors
    [A,D]=eigs(K/N,model.q,'LM',opts);
    % scale eigenvectors to unit length by scaling coefficients
    % ( 'real' should not be necessary, but 'eigs' ... )
    d=real(diag(D)).^(-0.5);
    A=A*diag(d);
    % get principal components
    X=A'*K;
else
    [d,N]=size(model.Y);
    Yc=model.Y-repmat(sum(model.Y,2)/N,1,N);
    opts.disp=0;
    [V,D]=eigs(Yc*Yc'/N,model.q,'LM',opts);
    X=V'*Yc;
end

% scale to variance 1
v=(N/mean(sum(X.^2,2))).^0.5;
X=X*v;
