function model = ukr_smooth(model,variant,eps_lb)
% function model = ukr_smooth(model,variant,eps_lb)
%
% High level wrapper for fitting an epsilon-insensitive
% UKR model, AFTER a possibly unsmooth LOO-CV manifold
% has been trained.
%
% model     - readily trained UKR model (LOO-CV)
% variant   - either 1 (epsilon per dimension) 
%                or  2 (epsilon for eucliden distance)
% eps_lb    - desired tolerance (lower bound for error)

switch variant
   case 1;
      model.eps = ukr_aux_eps(model,eps_lb);
      model.loss = 'Eps';
   case 2;
      model.eps = ukr_aux_eps2(model,eps_lb);
      model.loss = 'Eps2';
   otherwise;
      error '"variant" must either be 1 or 2';
end

model

model.Xcv = model.X;
model.X = ukr_opt_eps(@ukr_ext_penalty,model.Xcv,100,model);

